/************************************ <lalVerbatim file="PrintTimeSeriesCV">
Author: J.T. Whelan
$Id: PrintTimeSeries.m4,v 1.1 2000/11/06 21:11:22 jolien Exp $
************************************* </lalVerbatim> */

/* <lalLaTeX>

\subsection{Module \texttt{PrintTimeSeries.c}}
\label{ss:PrintTimeSeries.c}

Print a $\langle\mbox{datatype}\rangle$TimeSeries object into a
file.  For use in non-production and test code only.

\subsubsection*{Prototypes}
\vspace{0.1in}
\input{PrintTimeSeriesCP}
\index{\verb&LALZPrintTimeSeries()&}
\index{\verb&LALCPrintTimeSeries()&}
\index{\verb&LALDPrintTimeSeries()&}
\index{\verb&LALSPrintTimeSeries()&}
\index{\verb&LALI2PrintTimeSeries()&}
\index{\verb&LALI4PrintTimeSeries()&}
\index{\verb&LALI8PrintTimeSeries()&}
\index{\verb&LALU2PrintTimeSeries()&}
\index{\verb&LALU4PrintTimeSeries()&}
\index{\verb&LALU8PrintTimeSeries()&}
\index{\verb&LALPrintTimeSeries()&}

\subsubsection*{Description}

Each member of this family of functions prints the elements of
$\langle\mbox{datatype}\rangle$\verb+TimeSeries+ into a file.  Note:
the file name is specified using a character string.  This function is
for debugging use only: its arguments do not conform to LAL standards
so it should not be used in any real analysis codes.

\subsubsection*{Algorithm}

\subsubsection*{Uses}

\subsubsection*{Notes}

This function's arguments do not conform to the LAL spec.  For this
reason it should only be used for debugging purposes in test
functions, not in any production code.

Additionally, since printf cannot handle INT8 as integers, the
functions \verb&LALI8PrintTimeSeries()& and
\verb&LALU8PrintTimeSeries()& use a typecast to REAL8 and are thus
only valid for numbers between around $-10^{15}$ and $10^{15}$.

The output  is three or four space-separated columns: the first
column is the time of the beginning of the time series, in seconds
after the GPS reference epoch (1980 January 6), and is the same for
each entry; the second column is the time into the series (so that the
sum of the first two columns is the time of the entry in question);
for real and integer time series, the third column is the value of the
series; for complex time series, the third column is the real part and
the fourth the imaginary part of the value.

\vfill{\footnotesize\input{PrintTimeSeriesCV}}

</lalLaTeX> */


#include "LALStdlib.h"
#include <stdio.h>
#include "LALDatatypes.h"
#include "PrintFTSeries.h"

/* <lalVerbatim file="PrintTimeSeriesNRCSID"> */
NRCSID( PRINTTIMESERIESC, "$Id: PrintTimeSeries.m4,v 1.1 2000/11/06 21:11:22 jolien Exp $" );
/* </lalVerbatim> */






























/* <lalVerbatim file="PrintTimeSeriesCP"> */
void LALZPrintTimeSeries ( COMPLEX16TimeSeries *series, const CHAR *filename ) 
{ /* </lalVerbatim> */
  REAL8 t;
  REAL8 t0;
  COMPLEX16 *data;
  FILE *fp;
  COMPLEX16 *endOfSeq;

  if (series==NULL) return;

  /* *(series->data) is a COMPLEX16Sequence */
  /* series->data->data is a pointer to COMPLEX16 */

  /* Make a COMPLEX16 pointer which points to the first memory address not
   * belonging to the sequence
   */

  endOfSeq = series->data->data + series->data->length;

  /* open output file */
  fp=fopen(filename,"w");
	t0 = series->epoch.gpsSeconds + 1e-9 * series->epoch.gpsNanoSeconds;
  for ( t = 0,
        data = series->data->data;
        data < endOfSeq;
        t += series->deltaT, ++data )
  {
    fprintf(fp,"%g %g %g %g\n",t0,t,data->re,data->im
           );
  }	

  fclose(fp);

  return;
}































/* <lalVerbatim file="PrintTimeSeriesCP"> */
void LALCPrintTimeSeries ( COMPLEX8TimeSeries *series, const CHAR *filename ) 
{ /* </lalVerbatim> */
  REAL8 t;
  REAL8 t0;
  COMPLEX8 *data;
  FILE *fp;
  COMPLEX8 *endOfSeq;

  if (series==NULL) return;

  /* *(series->data) is a COMPLEX8Sequence */
  /* series->data->data is a pointer to COMPLEX8 */

  /* Make a COMPLEX8 pointer which points to the first memory address not
   * belonging to the sequence
   */

  endOfSeq = series->data->data + series->data->length;

  /* open output file */
  fp=fopen(filename,"w");
	t0 = series->epoch.gpsSeconds + 1e-9 * series->epoch.gpsNanoSeconds;
  for ( t = 0,
        data = series->data->data;
        data < endOfSeq;
        t += series->deltaT, ++data )
  {
    fprintf(fp,"%g %g %g %g\n",t0,t,data->re,data->im
           );
  }	

  fclose(fp);

  return;
}































/* <lalVerbatim file="PrintTimeSeriesCP"> */
void LALDPrintTimeSeries ( REAL8TimeSeries *series, const CHAR *filename ) 
{ /* </lalVerbatim> */
  REAL8 t;
  REAL8 t0;
  REAL8 *data;
  FILE *fp;
  REAL8 *endOfSeq;

  if (series==NULL) return;

  /* *(series->data) is a REAL8Sequence */
  /* series->data->data is a pointer to REAL8 */

  /* Make a REAL8 pointer which points to the first memory address not
   * belonging to the sequence
   */

  endOfSeq = series->data->data + series->data->length;

  /* open output file */
  fp=fopen(filename,"w");
	t0 = series->epoch.gpsSeconds + 1e-9 * series->epoch.gpsNanoSeconds;
  for ( t = 0,
        data = series->data->data;
        data < endOfSeq;
        t += series->deltaT, ++data )
  {
    fprintf(fp,"%g %g %g\n",t0,t,*data
           );
  }	

  fclose(fp);

  return;
}































/* <lalVerbatim file="PrintTimeSeriesCP"> */
void LALSPrintTimeSeries ( REAL4TimeSeries *series, const CHAR *filename ) 
{ /* </lalVerbatim> */
  REAL8 t;
  REAL8 t0;
  REAL4 *data;
  FILE *fp;
  REAL4 *endOfSeq;

  if (series==NULL) return;

  /* *(series->data) is a REAL4Sequence */
  /* series->data->data is a pointer to REAL4 */

  /* Make a REAL4 pointer which points to the first memory address not
   * belonging to the sequence
   */

  endOfSeq = series->data->data + series->data->length;

  /* open output file */
  fp=fopen(filename,"w");
	t0 = series->epoch.gpsSeconds + 1e-9 * series->epoch.gpsNanoSeconds;
  for ( t = 0,
        data = series->data->data;
        data < endOfSeq;
        t += series->deltaT, ++data )
  {
    fprintf(fp,"%g %g %g\n",t0,t,*data
           );
  }	

  fclose(fp);

  return;
}































/* <lalVerbatim file="PrintTimeSeriesCP"> */
void LALI2PrintTimeSeries ( INT2TimeSeries *series, const CHAR *filename ) 
{ /* </lalVerbatim> */
  REAL8 t;
  REAL8 t0;
  INT2 *data;
  FILE *fp;
  INT2 *endOfSeq;

  if (series==NULL) return;

  /* *(series->data) is a INT2Sequence */
  /* series->data->data is a pointer to INT2 */

  /* Make a INT2 pointer which points to the first memory address not
   * belonging to the sequence
   */

  endOfSeq = series->data->data + series->data->length;

  /* open output file */
  fp=fopen(filename,"w");
	t0 = series->epoch.gpsSeconds + 1e-9 * series->epoch.gpsNanoSeconds;
  for ( t = 0,
        data = series->data->data;
        data < endOfSeq;
        t += series->deltaT, ++data )
  {
    fprintf(fp,"%g %g %i\n",t0,t,*data
           );
  }	

  fclose(fp);

  return;
}































/* <lalVerbatim file="PrintTimeSeriesCP"> */
void LALI4PrintTimeSeries ( INT4TimeSeries *series, const CHAR *filename ) 
{ /* </lalVerbatim> */
  REAL8 t;
  REAL8 t0;
  INT4 *data;
  FILE *fp;
  INT4 *endOfSeq;

  if (series==NULL) return;

  /* *(series->data) is a INT4Sequence */
  /* series->data->data is a pointer to INT4 */

  /* Make a INT4 pointer which points to the first memory address not
   * belonging to the sequence
   */

  endOfSeq = series->data->data + series->data->length;

  /* open output file */
  fp=fopen(filename,"w");
	t0 = series->epoch.gpsSeconds + 1e-9 * series->epoch.gpsNanoSeconds;
  for ( t = 0,
        data = series->data->data;
        data < endOfSeq;
        t += series->deltaT, ++data )
  {
    fprintf(fp,"%g %g %i\n",t0,t,*data
           );
  }	

  fclose(fp);

  return;
}































/* <lalVerbatim file="PrintTimeSeriesCP"> */
void LALI8PrintTimeSeries ( INT8TimeSeries *series, const CHAR *filename ) 
{ /* </lalVerbatim> */
  REAL8 t;
  REAL8 t0;
  INT8 *data;
  FILE *fp;
  INT8 *endOfSeq;

  if (series==NULL) return;

  /* *(series->data) is a INT8Sequence */
  /* series->data->data is a pointer to INT8 */

  /* Make a INT8 pointer which points to the first memory address not
   * belonging to the sequence
   */

  endOfSeq = series->data->data + series->data->length;

  /* open output file */
  fp=fopen(filename,"w");
	t0 = series->epoch.gpsSeconds + 1e-9 * series->epoch.gpsNanoSeconds;
  for ( t = 0,
        data = series->data->data;
        data < endOfSeq;
        t += series->deltaT, ++data )
  {
    fprintf(fp,"%g %g %0.0f\n",t0,t,(REAL8) *data
           );
  }	

  fclose(fp);

  return;
}

/* Note that LALI8PrintTimeSeries does a typecast to REAL8 and is thus
 * inaccurate for numbers >~ 1e15 
 */






























/* <lalVerbatim file="PrintTimeSeriesCP"> */
void LALU2PrintTimeSeries ( UINT2TimeSeries *series, const CHAR *filename ) 
{ /* </lalVerbatim> */
  REAL8 t;
  REAL8 t0;
  UINT2 *data;
  FILE *fp;
  UINT2 *endOfSeq;

  if (series==NULL) return;

  /* *(series->data) is a UINT2Sequence */
  /* series->data->data is a pointer to UINT2 */

  /* Make a UINT2 pointer which points to the first memory address not
   * belonging to the sequence
   */

  endOfSeq = series->data->data + series->data->length;

  /* open output file */
  fp=fopen(filename,"w");
	t0 = series->epoch.gpsSeconds + 1e-9 * series->epoch.gpsNanoSeconds;
  for ( t = 0,
        data = series->data->data;
        data < endOfSeq;
        t += series->deltaT, ++data )
  {
    fprintf(fp,"%g %g %i\n",t0,t,*data
           );
  }	

  fclose(fp);

  return;
}































/* <lalVerbatim file="PrintTimeSeriesCP"> */
void LALU4PrintTimeSeries ( UINT4TimeSeries *series, const CHAR *filename ) 
{ /* </lalVerbatim> */
  REAL8 t;
  REAL8 t0;
  UINT4 *data;
  FILE *fp;
  UINT4 *endOfSeq;

  if (series==NULL) return;

  /* *(series->data) is a UINT4Sequence */
  /* series->data->data is a pointer to UINT4 */

  /* Make a UINT4 pointer which points to the first memory address not
   * belonging to the sequence
   */

  endOfSeq = series->data->data + series->data->length;

  /* open output file */
  fp=fopen(filename,"w");
	t0 = series->epoch.gpsSeconds + 1e-9 * series->epoch.gpsNanoSeconds;
  for ( t = 0,
        data = series->data->data;
        data < endOfSeq;
        t += series->deltaT, ++data )
  {
    fprintf(fp,"%g %g %i\n",t0,t,*data
           );
  }	

  fclose(fp);

  return;
}































/* <lalVerbatim file="PrintTimeSeriesCP"> */
void LALU8PrintTimeSeries ( UINT8TimeSeries *series, const CHAR *filename ) 
{ /* </lalVerbatim> */
  REAL8 t;
  REAL8 t0;
  UINT8 *data;
  FILE *fp;
  UINT8 *endOfSeq;

  if (series==NULL) return;

  /* *(series->data) is a UINT8Sequence */
  /* series->data->data is a pointer to UINT8 */

  /* Make a UINT8 pointer which points to the first memory address not
   * belonging to the sequence
   */

  endOfSeq = series->data->data + series->data->length;

  /* open output file */
  fp=fopen(filename,"w");
	t0 = series->epoch.gpsSeconds + 1e-9 * series->epoch.gpsNanoSeconds;
  for ( t = 0,
        data = series->data->data;
        data < endOfSeq;
        t += series->deltaT, ++data )
  {
    fprintf(fp,"%g %g %0.0f\n",t0,t,(REAL8) *data
           );
  }	

  fclose(fp);

  return;
}

/* Note that LALU8PrintTimeSeries does a typecast to REAL8 and is thus
 * inaccurate for numbers >~ 1e15 
 */






























/* <lalVerbatim file="PrintTimeSeriesCP"> */
void LALPrintTimeSeries ( REAL4TimeSeries *series, const CHAR *filename ) 
{ /* </lalVerbatim> */
  REAL8 t;
  REAL8 t0;
  REAL4 *data;
  FILE *fp;
  REAL4 *endOfSeq;

  if (series==NULL) return;

  /* *(series->data) is a REAL4Sequence */
  /* series->data->data is a pointer to REAL4 */

  /* Make a REAL4 pointer which points to the first memory address not
   * belonging to the sequence
   */

  endOfSeq = series->data->data + series->data->length;

  /* open output file */
  fp=fopen(filename,"w");
	t0 = series->epoch.gpsSeconds + 1e-9 * series->epoch.gpsNanoSeconds;
  for ( t = 0,
        data = series->data->data;
        data < endOfSeq;
        t += series->deltaT, ++data )
  {
    fprintf(fp,"%g %g %f\n",t0,t,*data
           );
  }	

  fclose(fp);

  return;
}

